{% extends 'allianceauth/base-bs5.html' %}

{% block header_nav_brand %}
    CharLink
{% endblock header_nav_brand %}

{% block header_nav_collapse_left %}
    {% if is_auditor %}
        {% include "charlink/menu-left.html" %}
    {% endif %}
{% endblock header_nav_collapse_left %}

{% block header_nav_collapse_right %}
    {% if is_auditor %}
        {% include "charlink/menu-right.html" %}
    {% endif %}
{% endblock header_nav_collapse_right %}

{% block content %}
    <div class="col-lg-12">
        {% block charlink_page_header %}{% endblock charlink_page_header %}
        <div class="container-fluid mt-3 px-0">
            {% block charlink_content %}{% endblock charlink_content %}
        </div>
    </div>
{% endblock content %}